@@ -43,107 +43,47 @@ code, you can add a second except for a specific subclass of an error:
             return e
 
 """
-import sys
+from __future__ import annotations
+
 import typing as t
-import warnings
 from datetime import datetime
-from html import escape
+
+from markupsafe import escape
+from markupsafe import Markup
 
 from ._internal import _get_environ
 
 if t.TYPE_CHECKING:
-    import typing_extensions as te
     from _typeshed.wsgi import StartResponse
     from _typeshed.wsgi import WSGIEnvironment
     from .datastructures import WWWAuthenticate
     from .sansio.response import Response
-    from .wrappers.request import Request as WSGIRequest  # noqa: F401
-    from .wrappers.response import Response as WSGIResponse  # noqa: F401
+    from .wrappers.request import Request as WSGIRequest
+    from .wrappers.response import Response as WSGIResponse
 
 
 class HTTPException(Exception):
     """The base class for all HTTP exceptions. This exception can be called as a WSGI
     application to render a default error page or you can catch the subclasses
     of it independently and render nicer error messages.
+
+    .. versionchanged:: 2.1
+        Removed the ``wrap`` class method.
     """
 
-    code: t.Optional[int] = None
-    description: t.Optional[str] = None
+    code: int | None = None
+    description: str | None = None
 
     def __init__(
         self,
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
+        description: str | None = None,
+        response: Response | None = None,
     ) -> None:
         super().__init__()
         if description is not None:
             self.description = description
         self.response = response
 
-    @classmethod
-    def wrap(
-        cls, exception: t.Type[BaseException], name: t.Optional[str] = None
-    ) -> t.Type["HTTPException"]:
-        """Create an exception that is a subclass of the calling HTTP
-        exception and the ``exception`` argument.
-
-        The first argument to the class will be passed to the
-        wrapped ``exception``, the rest to the HTTP exception. If
-        ``e.args`` is not empty and ``e.show_exception`` is ``True``,
-        the wrapped exception message is added to the HTTP error
-        description.
-
-        .. deprecated:: 2.0
-            Will be removed in Werkzeug 2.1. Create a subclass manually
-            instead.
-
-        .. versionchanged:: 0.15.5
-            The ``show_exception`` attribute controls whether the
-            description includes the wrapped exception message.
-
-        .. versionchanged:: 0.15.0
-            The description includes the wrapped exception message.
-        """
-        warnings.warn(
-            "'HTTPException.wrap' is deprecated and will be removed in"
-            " Werkzeug 2.1. Create a subclass manually instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-
-        class newcls(cls, exception):  # type: ignore
-            _description = cls.description
-            show_exception = False
-
-            def __init__(
-                self, arg: t.Optional[t.Any] = None, *args: t.Any, **kwargs: t.Any
-            ) -> None:
-                super().__init__(*args, **kwargs)
-
-                if arg is None:
-                    exception.__init__(self)
-                else:
-                    exception.__init__(self, arg)
-
-            @property
-            def description(self) -> str:
-                if self.show_exception:
-                    return (
-                        f"{self._description}\n"
-                        f"{exception.__name__}: {exception.__str__(self)}"
-                    )
-
-                return self._description  # type: ignore
-
-            @description.setter
-            def description(self, value: str) -> None:
-                self._description = value
-
-        newcls.__module__ = sys._getframe(1).f_globals["__name__"]
-        name = name or cls.__name__ + exception.__name__
-        newcls.__name__ = newcls.__qualname__ = name
-        return newcls
-
     @property
     def name(self) -> str:
         """The status name."""
@@ -153,28 +93,27 @@ class HTTPException(Exception):
 
     def get_description(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
     ) -> str:
         """Get the description."""
         if self.description is None:
             description = ""
-        elif not isinstance(self.description, str):
-            description = str(self.description)
         else:
             description = self.description
 
-        description = escape(description).replace("\n", "<br>")
+        description = escape(description).replace("\n", Markup("<br>"))
         return f"<p>{description}</p>"
 
     def get_body(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
     ) -> str:
         """Get the HTML body."""
         return (
-            '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">\n'
+            "<!doctype html>\n"
+            "<html lang=en>\n"
             f"<title>{self.code} {escape(self.name)}</title>\n"
             f"<h1>{escape(self.name)}</h1>\n"
             f"{self.get_description(environ)}\n"
@@ -182,17 +121,17 @@ class HTTPException(Exception):
 
     def get_headers(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
-    ) -> t.List[t.Tuple[str, str]]:
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
+    ) -> list[tuple[str, str]]:
         """Get a list of headers."""
         return [("Content-Type", "text/html; charset=utf-8")]
 
     def get_response(
         self,
-        environ: t.Optional[t.Union["WSGIEnvironment", "WSGIRequest"]] = None,
-        scope: t.Optional[dict] = None,
-    ) -> "Response":
+        environ: WSGIEnvironment | WSGIRequest | None = None,
+        scope: dict | None = None,
+    ) -> Response:
         """Get a response object.  If one was passed to the exception
         it's returned directly.
 
@@ -211,7 +150,7 @@ class HTTPException(Exception):
         return WSGIResponse(self.get_body(environ, scope), self.code, headers)
 
     def __call__(
-        self, environ: "WSGIEnvironment", start_response: "StartResponse"
+        self, environ: WSGIEnvironment, start_response: StartResponse
     ) -> t.Iterable[bytes]:
         """Call the exception as WSGI application.
 
@@ -256,7 +195,7 @@ class BadRequestKeyError(BadRequest, KeyError):
     #: useful in a debug mode.
     show_exception = False
 
-    def __init__(self, arg: t.Optional[str] = None, *args: t.Any, **kwargs: t.Any):
+    def __init__(self, arg: str | None = None, *args: t.Any, **kwargs: t.Any):
         super().__init__(*args, **kwargs)
 
         if arg is None:
@@ -265,7 +204,7 @@ class BadRequestKeyError(BadRequest, KeyError):
             KeyError.__init__(self, arg)
 
     @property  # type: ignore
-    def description(self) -> str:  # type: ignore
+    def description(self) -> str:
         if self.show_exception:
             return (
                 f"{self._description}\n"
@@ -357,11 +296,9 @@ class Unauthorized(HTTPException):
 
     def __init__(
         self,
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
-        www_authenticate: t.Optional[
-            t.Union["WWWAuthenticate", t.Iterable["WWWAuthenticate"]]
-        ] = None,
+        description: str | None = None,
+        response: Response | None = None,
+        www_authenticate: None | (WWWAuthenticate | t.Iterable[WWWAuthenticate]) = None,
     ) -> None:
         super().__init__(description, response)
 
@@ -374,9 +311,9 @@ class Unauthorized(HTTPException):
 
     def get_headers(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
-    ) -> t.List[t.Tuple[str, str]]:
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
+    ) -> list[tuple[str, str]]:
         headers = super().get_headers(environ, scope)
         if self.www_authenticate:
             headers.extend(("WWW-Authenticate", str(x)) for x in self.www_authenticate)
@@ -427,9 +364,9 @@ class MethodNotAllowed(HTTPException):
 
     def __init__(
         self,
-        valid_methods: t.Optional[t.Iterable[str]] = None,
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
+        valid_methods: t.Iterable[str] | None = None,
+        description: str | None = None,
+        response: Response | None = None,
     ) -> None:
         """Takes an optional list of valid http methods
         starting with werkzeug 0.3 the list will be mandatory."""
@@ -438,9 +375,9 @@ class MethodNotAllowed(HTTPException):
 
     def get_headers(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
-    ) -> t.List[t.Tuple[str, str]]:
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
+    ) -> list[tuple[str, str]]:
         headers = super().get_headers(environ, scope)
         if self.valid_methods:
             headers.append(("Allow", ", ".join(self.valid_methods)))
@@ -584,10 +521,10 @@ class RequestedRangeNotSatisfiable(HTTPException):
 
     def __init__(
         self,
-        length: t.Optional[int] = None,
+        length: int | None = None,
         units: str = "bytes",
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
+        description: str | None = None,
+        response: Response | None = None,
     ) -> None:
         """Takes an optional `Content-Range` header value based on ``length``
         parameter.
@@ -598,9 +535,9 @@ class RequestedRangeNotSatisfiable(HTTPException):
 
     def get_headers(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
-    ) -> t.List[t.Tuple[str, str]]:
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
+    ) -> list[tuple[str, str]]:
         headers = super().get_headers(environ, scope)
         if self.length is not None:
             headers.append(("Content-Range", f"{self.units} */{self.length}"))
@@ -698,18 +635,18 @@ class _RetryAfter(HTTPException):
 
     def __init__(
         self,
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
-        retry_after: t.Optional[t.Union[datetime, int]] = None,
+        description: str | None = None,
+        response: Response | None = None,
+        retry_after: datetime | int | None = None,
     ) -> None:
         super().__init__(description, response)
         self.retry_after = retry_after
 
     def get_headers(
         self,
-        environ: t.Optional["WSGIEnvironment"] = None,
-        scope: t.Optional[dict] = None,
-    ) -> t.List[t.Tuple[str, str]]:
+        environ: WSGIEnvironment | None = None,
+        scope: dict | None = None,
+    ) -> list[tuple[str, str]]:
         headers = super().get_headers(environ, scope)
 
         if self.retry_after:
@@ -788,9 +725,9 @@ class InternalServerError(HTTPException):
 
     def __init__(
         self,
-        description: t.Optional[str] = None,
-        response: t.Optional["Response"] = None,
-        original_exception: t.Optional[BaseException] = None,
+        description: str | None = None,
+        response: Response | None = None,
+        original_exception: BaseException | None = None,
     ) -> None:
         #: The original exception that caused this 500 error. Can be
         #: used by frameworks to provide context when handling
@@ -869,7 +806,7 @@ class HTTPVersionNotSupported(HTTPException):
     )
 
 
-default_exceptions: t.Dict[int, t.Type[HTTPException]] = {}
+default_exceptions: dict[int, type[HTTPException]] = {}
 
 
 def _find_exceptions() -> None:
@@ -901,8 +838,8 @@ class Aborter:
 
     def __init__(
         self,
-        mapping: t.Optional[t.Dict[int, t.Type[HTTPException]]] = None,
-        extra: t.Optional[t.Dict[int, t.Type[HTTPException]]] = None,
+        mapping: dict[int, type[HTTPException]] | None = None,
+        extra: dict[int, type[HTTPException]] | None = None,
     ) -> None:
         if mapping is None:
             mapping = default_exceptions
@@ -911,8 +848,8 @@ class Aborter:
             self.mapping.update(extra)
 
     def __call__(
-        self, code: t.Union[int, "Response"], *args: t.Any, **kwargs: t.Any
-    ) -> "te.NoReturn":
+        self, code: int | Response, *args: t.Any, **kwargs: t.Any
+    ) -> t.NoReturn:
         from .sansio.response import Response
 
         if isinstance(code, Response):
@@ -924,9 +861,7 @@ class Aborter:
         raise self.mapping[code](*args, **kwargs)
 
 
-def abort(
-    status: t.Union[int, "Response"], *args: t.Any, **kwargs: t.Any
-) -> "te.NoReturn":
+def abort(status: int | Response, *args: t.Any, **kwargs: t.Any) -> t.NoReturn:
     """Raises an :py:exc:`HTTPException` for the given status code or WSGI
     application.
 
